//  DirichletDomain-GL.hpp
//
//       YOU DO NOT NEED TO READ THIS FILE TO UNDERSTAND
//             COMPUTER GRAPHICS IN CURVED SPACES.
//
//      It serves only to provide some interesting content
//           in the complete version of the program.

#pragma once

#include "Macros-GL.h"

//  Opaque typedefs for winged edge polyhedron.
typedef struct WEVertex         WEVertex;
typedef struct WEEdge           WEEdge;
typedef struct WEFace           WEFace;
typedef struct WEPolyhedron     WEPolyhedron;

//  A Direction is a vector whose direction is
//  well defined but whose length is arbitrary.
//  That is, two Directions are considered equivalent
//  iff they are positive multiples of one another.
//  Note:  a Direction and its negative are *not* equivalent.
typedef double  Direction[4];

//  For most purposes the Dirichlet domain doesn't need
//  to know the SpaceType, but I wanted to sneak it in
//  to normalize the beam width consistently.
//  I know it's a kludge, but I'm keeping the original
//  typedef SpaceType in CurvedSpaces.cpp for the sake of
//  CurvedSpacesSimple.cpp, which won't use Dirichlet domains
//  and will keep a purely local definition of SpaceType.
typedef enum
{
    SpaceSphericalDD = 0,
    SpaceFlatDD,
    SpaceHyperbolicDD
} SpaceTypeDD;


class CDirichletDomain
{
public:

                    CDirichletDomain(   int         aNumMatrices,
                                        Matrix      *aMatrixList,
                                        SpaceTypeDD aSpaceType);
    virtual         ~CDirichletDomain(void);

    virtual void    CenterCamera(Matrix *aCameraPosition);

    virtual void    RenderBeams(float aFogFactor, GLuint aTexture);

protected:

    WEPolyhedron    *itsPolyhedron;

    void    FreePolyhedron(void);
    void    MakeCube(void);
    void    MakeEmptyPolyhedron(void);
    void    SetInitialCube(void);
    void    SlicePolyhedron(int aNumMatrices, Matrix *aMatrixList);
    void    IntersectWithHalfspace(Matrix *aMatrix, double *aWorstError);
    void    SplitEdge(WEEdge *anOldEdge, Direction aCutPoint);
    void    CutFaceIfNecessary(WEFace *aFace);
    void    AllEdgesCounterclockwise(WEFace *aFace);
    void    RedirectEdge(WEEdge *anEdge);
    void    InstallNewFace(Matrix *aMatrix, Direction aNormal);
    BOOL    TemporaryFacesRemain(void);
    void    AssignMates(void);
    void    CountCells(void);
    void    VerifyFaces(void);
    void    AssignHues(void);
    void    PrepareBeams(SpaceTypeDD aSpaceType);
    void    InterpolateEdge(double aNearPoint[4], double aFarPoint[4], SpaceTypeDD aSpaceType, GLfloat aBeamVertex[3]);
    void    Normalize(double aVector[4], SpaceTypeDD aSpaceType);
    double  InnerProduct(double u[4], double v[4], SpaceTypeDD aSpaceType);
    double  ArcCosh(double x);
};
