//  SceneryObjectCube-GL.cpp

#include "SceneryObjectCube-GL.hpp"

#define DEGREES_PER_SECOND  45
#define CUBE_SIZE           0.05f



CSceneryObjectCube::CSceneryObjectCube(void)
{
    itsRotationAngle = 0.0;
}


void CSceneryObjectCube::Update(double anElapsedTime)
{
    itsRotationAngle += DEGREES_PER_SECOND * anElapsedTime;

    if (itsRotationAngle > 360.0)
        itsRotationAngle -= 360.0;
}


void CSceneryObjectCube::Render(void)
{
    int i;

    static GLfloat theVertices[8*3] =
    {
        -CUBE_SIZE, -CUBE_SIZE, -CUBE_SIZE,
        -CUBE_SIZE, +CUBE_SIZE, -CUBE_SIZE,
        +CUBE_SIZE, +CUBE_SIZE, -CUBE_SIZE,
        +CUBE_SIZE, -CUBE_SIZE, -CUBE_SIZE,
        -CUBE_SIZE, -CUBE_SIZE, +CUBE_SIZE,
        -CUBE_SIZE, +CUBE_SIZE, +CUBE_SIZE,
        +CUBE_SIZE, +CUBE_SIZE, +CUBE_SIZE,
        +CUBE_SIZE, -CUBE_SIZE, +CUBE_SIZE
    };
    static GLuint theFaces[6][4] =
    {
        {0, 1, 2, 3},
        {4, 7, 6, 5},
        {0, 3, 7, 4},
        {1, 5, 6, 2},
        {0, 4, 5, 1},
        {3, 2, 6, 7}
    };
    static GLfloat theColors[6][3] =
    {
        {1, 0, 0},
        {0, 1, 1},
        {0, 1, 0},
        {1, 0, 1},
        {0, 0, 1},
        {1, 1, 0}
    };

    //  Get ready.
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();

    //  Rotate the cube about the y-axis, then move it to its position.
    //  (Note:  OpenGL accepts these instructions in reverse order.)
    glMultMatrixf((float *) &itsPosition.m);
    glRotated(itsRotationAngle, 0, 1, 0);

    //  Draw the cube.
    glEnableClientState(GL_VERTEX_ARRAY);
    glVertexPointer(3, GL_FLOAT, 0, theVertices);
    for (i = 0; i < 6; i++)
    {
        glColor3fv(theColors[i]);
        glDrawElements(GL_QUADS, 4, GL_UNSIGNED_INT, theFaces[i]);
    }
    glDisableClientState(GL_VERTEX_ARRAY);

    //  Clean up.
    glPopMatrix();
}
